from typing import Optional
from mako.template import Template as MakoTemplate
from jinja2 import Template as Jinja2Template
from turandot import TurandotConversionException
from turandot.model import ConverterBase, TemplatingEngine, QueueMessage, ConversionJob


class ApplyTemplate(ConverterBase):
    """Apply Jinja or Mako template in current conversion process"""

    DESC: str = "Applying template"
    PREFIX: Optional[str] = "posttmpl_"
    POSTFIX: Optional[str] = ".html"
    SAVABLE: bool = True

    def process_step(self) -> ConversionJob:
        engine = self._get_engine()
        self.conversion_job.current_step.content = self._apply_template(engine)
        return self.conversion_job

    def _get_engine(self) -> TemplatingEngine:
        """Read engine config from config file & database"""
        engine = str(self.conversion_job.job_assets.template.metadata.get("engine", "")).lower()
        if engine.startswith("jinja"):
            if self.conversion_job.job_assets.template.allow_jinja:
                return TemplatingEngine.JINJA2
            else:
                msg = QueueMessage.warningmsg("WARNING: Template metadata suggest Jinja templating, but it is not allowed.")
                self.conversion_job.msgqueue.put(msg)
        elif engine.startswith("mako"):
            if self.conversion_job.job_assets.template.allow_mako:
                return TemplatingEngine.MAKO
            else:
                msg = QueueMessage.warningmsg("WARNING: Template metadata suggest Mako templating, but it is not allowed.")
                self.conversion_job.msgqueue.put(msg)
        return TemplatingEngine.NOTHING

    def _apply_template(self, engine: TemplatingEngine) -> str:
        """Process templating based on type"""
        if engine == TemplatingEngine.JINJA2:
            return self._apply_jinja()
        elif engine == TemplatingEngine.MAKO:
            return self._apply_mako()
        else:
            return self._apply_replace()

    def _apply_jinja(self) -> str:
        """Apply template for Jinja engine"""
        tmpl = Jinja2Template(self.conversion_job.job_assets.template.content)
        try:
            return tmpl.render(
                body=self.conversion_job.current_step.content,
                **self.conversion_job.job_assets.sourcefile.metadata
            )
        except (NameError, RuntimeError):
            raise TurandotConversionException("Jinja conversion error: Probably a missing variable value. Please double-check your YAML header.")

    def _apply_mako(self) -> str:
        """Apply template for Mako engine"""
        tmpl = MakoTemplate(self.conversion_job.job_assets.template.content)
        try:
            return tmpl.render(
                body=self.conversion_job.current_step.content,
                **self.conversion_job.job_assets.sourcefile.metadata
            )
        except (NameError, RuntimeError):
            raise TurandotConversionException("Mako conversion error: Probably a missing variable value. Please double-check your YAML header.")

    def _apply_replace(self) -> str:
        """Apply template for no engine"""
        placeholders = ["${body}", "{{body}}"]
        for i in placeholders:
            if i in self.conversion_job.job_assets.template.content:
                return self.conversion_job.job_assets.template.content.replace(
                    i, self.conversion_job.current_step.content
                )
        raise TurandotConversionException("Conversion error: No placeholder found in template. Try including \"{body}\" in your template file.")



from typing import Optional, Union
from pathlib import Path
import peewee
import zipfile
import yaml
import frontmatter
from yaml.scanner import ScannerError
from turandot import TurandotAssetException
from turandot.model import DatabaseAsset, ConfigModel
from turandot.model.db.dbmodels import Templates


class TemplateAsset(DatabaseAsset):
    """Conversion asset representing a conversion template"""

    DB_MODEL = Templates
    METAFILE: str = "meta.yaml"

    def __init__(self, path: str, allow_mako: bool, allow_jinja: bool, dbid: Optional[int] = None, expand: bool = False):
        self.allow_jinja: bool = allow_jinja
        self.allow_mako: bool = allow_mako
        self.zipped: bool = False
        self.metadata: dict = {}
        DatabaseAsset.__init__(self, path=path, dbid=dbid, expand=expand)

    def expand(self):
        self._read_path()
        self.zipped = zipfile.is_zipfile(self.path)
        self._read_meta()
        self._read_title()
        self._read_content()

    def _read_zipped_meta(self) -> dict:
        """Get metadata dict from a zipped template"""
        try:
            with zipfile.ZipFile(self.path) as f:
                yamlstr = f.read(TemplateAsset.METAFILE)
            return yaml.safe_load(yamlstr)
        except FileNotFoundError:
            raise TurandotAssetException("Metadata file not found")
        except ScannerError:
            raise TurandotAssetException("Template metadata file not readable")

    def _read_text_meta(self) -> dict:
        """Get metadata dict from a non-zipped template"""
        try:
            enc = ConfigModel().get_key(["general", "encoding"], default="utf8")
            with self.path.open('r', encoding=enc) as f:
                metatext = f.read()
            return yaml.safe_load(metatext)
        except FileNotFoundError:
            raise TurandotAssetException("Metadata file not found")
        except ScannerError:
            raise TurandotAssetException("Template metadata file not readable")

    def _read_meta(self):
        """Read metadata from template"""
        self.metadata = self._read_zipped_meta() if self.zipped else self._read_text_meta()

    def _read_title(self):
        self.title = self.metadata.get("title", TemplateAsset.TITLE_NOT_FOUND_MSG)

    def _read_content(self):
        self.content = self._read_zipped_content() if self.zipped else self._read_text_content()

    def _read_text_content(self) -> str:
        """Read template entry point content from a non-zipped template"""
        entrypoint = self.directory / self.metadata.get("entrypoint", None)
        if entrypoint is None:
            raise TurandotAssetException("No entrypoint for template specified")
        if not entrypoint.is_file():
            raise TurandotAssetException("Entrypoint file not found at {}".format(entrypoint))
        with entrypoint.open('r') as f:
            content = f.read()
        return content

    def _read_zipped_content(self) -> str:
        """Read template entry point content from a zipped template"""
        entrypoint = self.metadata.get("entrypoint", None)
        if entrypoint is None:
            raise TurandotAssetException("No entrypoint for template specified")
        try:
            with zipfile.ZipFile(self.path) as f:
                content = f.read(entrypoint)
            return content.decode()
        except FileNotFoundError:
            raise TurandotAssetException("Entrypoint file not found")

    def _to_orm(self) -> Templates:
        try:
            entry = Templates.get(dbid=self.dbid)
        except peewee.DoesNotExist:
            return Templates.create(path=self.path, allow_jinja=self.allow_jinja, allow_mako=self.allow_mako)
        entry.path = self.path
        entry.allow_jinja = self.allow_jinja
        entry.allow_mako = self.allow_mako
        return entry

    @classmethod
    def _from_orm(cls, model: Templates, expand: bool):
        return cls(path=model.path, dbid=model.dbid, allow_jinja=model.allow_jinja, allow_mako=model.allow_mako, expand=expand)



from turandot.ui import ExceptionCatcher, TurandotFrontend
from turandot.ttk.controllers import \
    SettingsController, DbDropdownController, ConverterController, ExceptionController, AboutController, \
    ExportController
from turandot.ttk.view import TurandotTtkView
from turandot.ttk import TkCatcher


class TurandotTtk(TurandotFrontend):
    """Assemble and draw TK frontend for the application"""

    def __init__(self):
        self.view = TurandotTtkView()
        # Controller needed for the exception catcher
        self.exception_controller = ExceptionController(self.view)
        catcher_strategy = TkCatcher(self.exception_controller)
        self.catcher = ExceptionCatcher()
        self.catcher.set_strategy(catcher_strategy)

    def _attach_controllers(self):
        """Attach all controllers to the view"""
        SettingsController(self.view)
        DbDropdownController(self.view)
        ConverterController(self.view)
        AboutController(self.view)
        ExportController(self.view)

    def run(self):
        """Run the TK frontend of the application"""
        self._attach_controllers()
        self.view.run()


if __name__ == "__main__":
    TurandotTtk().run()



from tkinter import *
from tkinter import ttk

from turandot.ttk.view.styles import TTKStyles
from turandot.ttk.view import ViewComponent


class SettingsTab(ViewComponent):
    """Create view process for 'Settings' tab"""

    def _create_config_file_opener(self, sframe: ttk.Frame):
        ttk.Label(sframe, text="Config file", style="h2.TLabel", padding=TTKStyles.get_padding().h2_top)\
            .grid(row=0, columnspan=2, sticky=(W, E))
        ttk.Label(sframe, text="Location:").grid(
            row=1, column=0, sticky=W, **TTKStyles.get_padding().left_labels
        )
        self.widgets["config_file_location_entry_value"] = StringVar()
        self.widgets["config_file_location_entry"] = ttk.Entry(
            sframe, textvariable=self.widgets["config_file_location_entry_value"], state="readonly"
        )
        self.widgets["config_file_location_entry"].grid(
            row=1, column=1, sticky=(W, E), **TTKStyles.get_padding().right_entries
        )
        self.widgets["config_dir_open_button"] = ttk.Button(sframe, text="Open folder")
        self.widgets["config_dir_open_button"].grid(
            row=2, columnspan=2, sticky=E, **TTKStyles.get_padding().right_entries
        )

    def _create_dynamic_settings_frame(self, sframe: ttk.Frame):
        self.widgets["dynamic_settings_container"] = ttk.Frame(sframe)
        self.widgets["dynamic_settings_container"].grid(row=3, columnspan=2, sticky=(W, E))
        self.widgets["dynamic_settings_container"].columnconfigure(0, weight=0)
        self.widgets["dynamic_settings_container"].columnconfigure(1, weight=1)

    def create(self, add_to: ttk.Frame) -> ttk.Frame:
        self.widgets["settings_container"] = ttk.Frame(add_to)
        self.widgets["settings_container"].grid(column=0, row=0, sticky=(N, S, W, E))
        self.widgets["settings_container"].columnconfigure(0, weight=0)
        self.widgets["settings_container"].columnconfigure(1, weight=1)
        self._create_config_file_opener(self.widgets["settings_container"])
        return self.widgets["settings_container"]



from turandot.model.datastructures.enums import ReferenceSource, TemplatingEngine, ConversionAlgorithm, OptionalStage, MessageType, Architecture, OpSys
from turandot.model.util import ModelUtils
from turandot.model.config.configdict import ConfigDict
from turandot.model.config.config import ConfigModel
from turandot.model.datastructures.dbasset import DatabaseAsset
from turandot.model.datastructures.tmplasset import TemplateAsset
from turandot.model.datastructures.cslasset import CslAsset
from turandot.model.datastructures.textasset import TextAsset
from turandot.model.datastructures.sourceasset import SourceAsset
from turandot.model.db.fpersist import FileSelectPersistApi
from turandot.model.datastructures.conversionjob import JobAssets, JobSettings, ConversionJob
from turandot.model.datastructures.queuemsg import QueueMessage
from turandot.model.datastructures.companiondata import CompanionData
from turandot.model.zoteroconnector import ZoteroConnector
from turandot.model.converter.converterbase import OptionalConverter
from turandot.model.converter.converterbase import ConverterBase
from turandot.model.converter.frontendstrategy import FrontendStrategy
from turandot.model.converter.gatherdata import GatherData
from turandot.model.converter.copytemplate import CopyTemplate
from turandot.model.converter.convtohtml import ConvertToHtml
from turandot.model.converter.applytemplate import ApplyTemplate
from turandot.model.converter.weasytopdf import WeasyprintToPdf
from turandot.model.converter.converterchain import ConverterChain
from turandot.model.converter.conversionprocessor import ConversionProcessor



import uuid
import multiprocessing
from typing import Optional, Union
from dataclasses import dataclass, astuple
from pathlib import Path
from turandot.model import TemplateAsset, CslAsset, TextAsset, SourceAsset, ConfigModel, ReferenceSource, ConversionAlgorithm, ConfigDict


@dataclass
class CurrentStep:
    """Data class to contain plain text of the document during conversion"""

    filename: str = ""
    content: str = ""

    def save_file(self, directory: Path):
        """Save step result to file for debugging"""
        with Path(directory / self.filename).open("w") as f:
            f.write(self.content)


@dataclass
class JobAssets:
    """Container class for all assets: source file, template, csl file"""
    sourcefile: Union[Path, str, SourceAsset]
    template: Union[int, TemplateAsset]
    cslfile: Optional[Union[int, CslAsset]] = None
    csljson: Optional[Union[Path, str, TextAsset]] = None

    def __iter__(self):
        return iter(astuple(self))


@dataclass
class JobSettings:
    """Container class for settings from GUI"""
    conversion_algorithm: ConversionAlgorithm
    reference_source: ReferenceSource
    zotero_lib_id: Optional[int] = None


class ConversionJob:
    """Data class containing all necessary information to perform a conversion"""

    def __init__(self, job_assets: JobAssets, job_settings: JobSettings, msgqueue: multiprocessing.Queue):
        self.job_assets: JobAssets = job_assets
        self.job_settings: JobSettings = job_settings
        self.msgqueue: multiprocessing.Queue = msgqueue
        self.current_step: CurrentStep = CurrentStep()
        self.conversion_id = str(uuid.uuid4())
        self.config: ConfigDict = ConfigModel().get_dict()



import pathlib
from distutils.core import setup
from setuptools import find_packages


here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')


setup(
    name='turandot',
    version='3.0.0b2',
    description='Turandot Markdown Converter',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="GPLv3",
    author='Martin Obrist',
    author_email='dev@obrist.email',
    url='https://turandot.readthedocs.io',
    project_urls={
        'Documentation': 'https://turandot.readthedocs.io',
        'Source Code': 'https://gitlab.com/dinuthehuman/turandot',
        'Issue Tracker': 'https://gitlab.com/dinuthehuman/turandot/-/issues'
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Text Processing :: Markup :: Markdown',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        "Programming Language :: Python :: 3.10",
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='markdown, citeproc',
    packages=find_packages(where='.', exclude=['tests', 'tasks']),
    python_requires='>=3.9, <4',
    include_package_data=True,
    setup_requires=['wheel'],
    install_requires=[
        "beautifulsoup4",
        "Mako",
        "Markdown",
        "python-frontmatter",
        "requests",
        "urllib3",
        "weasyprint>=53.0",
        "cssselect2",
        "jinja2",
        "markdown-katex",
        "colour",
        "peewee",
        "lxml",
        "pyyaml",
        "Pygments",
        "ruamel.yaml",
        "md_citeproc"
    ],
    extras_require={
        'tk': [
            'bidict',
            'tkhtmlview'
        ],
        'gtk': [
            'pygobject'
        ],
        'dev': ['gitpython', 'mkdocs', 'mkdocstrings', 'twine', 'pytest', 'nose', 'invoke'],
        'optional': ["gitpython", "qrcode"]
    }
)



from typing import Optional
from dataclasses import dataclass
from turandot.model import MessageType


@dataclass
class QueueMessage:
    """Data class for messages to be passed from backend to frontend"""
    type: MessageType
    copy: Optional[tuple] = None
    exception: Optional[Exception] = None
    traceback: Optional[str] = None
    warning: Optional[str] = None
    n_step: Optional[int] = None
    total_steps: Optional[int] = None
    step_desc: Optional[str] = None

    @classmethod
    def startedmsg(cls, total_steps: int):
        """Create message indicating that the conversion process has started"""
        return cls(
            type=MessageType.STARTED,
            total_steps=total_steps
        )

    @classmethod
    def copymsg(cls, v: tuple):
        """Create message indicating that a file has been created (to delete on cleanup"""
        return cls(
            type=MessageType.COPY,
            copy=v
        )

    @classmethod
    def warningmsg(cls, v):
        """Create message indicating that the conversion process emitted a warning"""
        return cls(
            type=MessageType.WARNING,
            warning=v
        )

    @classmethod
    def stepmsg(cls, n_step, total_steps, step_desc):
        """Create message indicating that a mandatory processor has finished"""
        return cls(
            type=MessageType.NEXT_STEP,
            n_step=n_step,
            total_steps=total_steps,
            step_desc=step_desc
        )

    @classmethod
    def exceptionmsg(cls, e, tb):
        """Create message indicating that the conversion process threw an exception"""
        return cls(
            type=MessageType.EXCEPTION,
            exception=e,
            traceback=tb
        )

    @classmethod
    def successmsg(cls):
        """Create message indicating that the conversion process has succeeded"""
        return cls(
            type=MessageType.SUCCESS
        )

    @classmethod
    def deathmsg(cls):
        """Create message indicating that the conversion process has died"""
        return cls(
            type=MessageType.DIED
        )



from abc import ABC, abstractmethod


class TurandotFrontend(ABC):
    """Abstract class for a GUI for the application"""

    @abstractmethod
    def run(self):
        """Draw frontend, run application"""
        pass



from turandot.ui import CatcherStrategy
from turandot.ttk.controllers import ExceptionController


class TkCatcher(CatcherStrategy):
    """Tk specific exception handling strategy"""

    def __init__(self, ctrl: ExceptionController):
        self.exc_controller = ctrl

    def handle_exception(self, e: Exception, tb: str):
        """Draw exception and traceback to Tk frontend"""
        self.exc_controller.draw_exception(e, tb)



from turandot.ttk.presentations.dropdownobservables import NotificationReason
from turandot.ttk.presentations.kvcombobox import KvCombobox
from turandot.ttk.presentations.dbfedpicker import DbFedFilePicker
from turandot.ttk.presentations.enumcombobox import EnumCombobox
from turandot.ttk.presentations.configpresentations import \
    TitleControl, TextControl, SpinControl, SwitchControl, TextEnumControl
from turandot.ttk.presentations.dropdownobservables import DatabaseDropdownObservable, TemplateObservable, CslObservable
from turandot.ttk.presentations.dropdownobservers import DropdownObserver, DatabaseDropdown, DatabaseNewOptDropdown



from abc import ABC, abstractmethod
from typing import Union, OrderedDict, Any


class ConfigDict(dict):
    """Overridden dict class with fancier getters/setters"""

    @staticmethod
    def _rec_ordered_to_dict(od: Any):
        """Convert a nested OrderedDict to ConfigDict recursively"""
        if not isinstance(od, (OrderedDict, dict)):
            return od
        for k, v in od.items():
            if isinstance(v, (OrderedDict, dict)):
                od[k] = ConfigDict._rec_ordered_to_dict(v)
        return ConfigDict(od)

    @classmethod
    def from_ordered(cls, od: Union[OrderedDict, dict]):
        """Create object from ruamel.yaml OrderedDict"""
        return cls(ConfigDict._rec_ordered_to_dict(od))

    def interlace(self, d: dict):
        """
        Override values with values from an incoming dict
        :param d: Dict containing the values to override
        :return: New ConfigDict with overridden values
        """
        interlaced = self | d
        return ConfigDict.from_ordered(interlaced)

    def get_key(self, key: Union[list, str], default=None):
        """
        Get nested key by key string or list of key strings
        :param key: key as string or list of keys for nested dicts
        :param default: default value to return if key is not found
        :return: value for key
        """
        if type(key) != list:
            return self.get(key, default)
        elif len(key) == 0:
            return self
        else:
            if key[0] not in self.keys():
                return default
            hay = self.get(key[0])
            if len(key[1:]) == 0:
                return hay
            elif isinstance(hay, ConfigDict):
                return hay.get_key(key[1:], default=default)
            return default



from typing import Optional, Callable, Union
import gi
gi.require_version('Gtk', '3.0')
from gi.repository.Gtk import ComboBoxText, ListStore
from turandot.model import DatabaseAsset
from turandot.ui import background
from turandot.gtk import guithread, DropdownPreSelections
from peewee import DoesNotExist


class DatabaseDropdown(ComboBoxText):
    """Database-fed Combobox widget"""

    NEW_OPT_TEXT = "- create new entry -"

    def _empty_callback(self):
        """Empty callback for initialization"""
        return

    def __init__(self, model: type(DatabaseAsset), new_option: bool = False):
        self.model = model
        self.new_option = new_option
        self.change_callback: Callable = self._empty_callback
        self.contains_ids: list[int] = []
        ComboBoxText.__init__(self)

    def set_change_callback(self, f: Callable):
        """Attach new function to call on change"""
        self.change_callback = f
        self.connect("changed", self.change_callback)

    @background
    def update(self, preselect: Union[DropdownPreSelections, int, type(None)] = None):
        """Update ComboBox from database"""
        preupdate = self.get_active_id()
        preupdate = None if preupdate is None else int(preupdate)
        elemls = self.model.get_all(expand=True)
        self.contains_ids = []
        store = ListStore(str, str)
        if self.new_option:
            store.append([DatabaseDropdown.NEW_OPT_TEXT, "0"])
        for i in elemls:
            self.contains_ids.append(i.dbid)
            store.append([i.title, str(i.dbid)])
        self._show_update(store, preselect, preupdate)

    @guithread
    def _show_update(
        self,
        store: ListStore,
        preselect: Union[DropdownPreSelections, int, type(None)] = None,
        preupdate_selection: Optional[int] = None
    ):
        self.set_model(store)
        if preselect is not None:
            self.set_selected_id(preselect, preupdate_selection)
            self.change_callback()

    def set_selected_id(
        self,
        selid: Union[DropdownPreSelections, int],
        preupdate_selection: Optional[int] = None
    ):
        """Set selected value with database id"""
        if selid == DropdownPreSelections.FIRST_ELEMENT:
            self.set_active(0)
        elif selid == DropdownPreSelections.SAME_ELEMENT:
            if preupdate_selection in self.contains_ids:
                self.set_active_id(str(preupdate_selection))
            else:
                self.set_active(0)
        else:
            if selid in self.contains_ids:
                self.set_active_id(str(selid))
            else:
                self.set_active(0)

    def get_selected_asset(self) -> Optional[DatabaseAsset]:
        """Get DatabaseAsset currently selected"""
        dbid = self.get_active_id()
        if dbid is None:
            return None
        dbid = int(dbid)
        try:
            entry = self.model.get(dbid, expand=True)
            return entry
        except DoesNotExist:
            return None



from enum import EnumMeta
from typing import Callable, Union
import gi
gi.require_version('Gtk', '3.0')
from gi.repository.Gtk import ComboBoxText, ListStore

from turandot.ui import EnumTranslations


class EnumDropdown(ComboBoxText):
    """Enum-based Combobox widget"""

    def _empty_callback(self):
        """Empty callback for initialization"""
        return

    def __init__(self, model: EnumMeta, text_based=False):
        self.text_based = text_based
        self.model = model
        ComboBoxText.__init__(self)
        self._create_options()
        self.change_callback: Callable = self._empty_callback

    def _create_options(self):
        """Set combobox options from enum values"""
        store = ListStore(str, str)
        for i in self.model:
            store.append([EnumTranslations.get(i), str(i.value)])
        self.set_model(store)
        self.set_active(0)

    def get_selected_value(self):
        """Get selected enum member"""
        if self.text_based:
            n = self.get_active_id()
        else:
            n = int(self.get_active_id())
        return self.model(n)

    def set_selected_value(self, active_id: Union[int, str]):
        """Set selected enum member from enum value"""
        self.set_active_id(active_id)

    def set_change_callback(self, f: Callable):
        """Attach new function to call on change"""
        self.change_callback = f
        self.connect("changed", self.change_callback)



from bs4 import BeautifulSoup
from turandot.model import OptionalConverter, ConversionJob


class TocPaginationContainers(OptionalConverter):
    """Optional converter to wrap page numbers in TOC into additional tags for easier styling"""

    def check_config(self, config: dict) -> bool:
        return bool(self.conversion_job.config.get_key(['opt_processors', 'toc_pagination_containers', 'enable'], default=False))

    def process_step(self) -> ConversionJob:
        soup = BeautifulSoup(self.conversion_job.current_step.content, features="html5lib")
        toc_tag = soup.find("div", {'class': 'toc'})
        if toc_tag is not None:
            toc_links = toc_tag.findChildren("a", recursive=True)
            for i in toc_links:
                container = soup.new_tag("a")
                container['href'] = i['href']
                container['class'] = "tocpagenr"
                i.insert_after(container)
        self.conversion_job.current_step.content = str(soup)
        return self.conversion_job



from tkinter import *
from tkinter import ttk

from turandot.ttk.view.styles import TTKStyles
from turandot.ttk.view import ViewComponent


class CslTab(ViewComponent):
    """View component for the 'csl' tab"""

    def _create_csl_file_selector(self, tframe: ttk.Frame):
        ttk.Label(tframe, text="Entry to edit:").grid(
            row=0, column=0, sticky=W, **TTKStyles.get_padding().left_labels
        )
        self.widgets["csl_editor_dropdown_frame"] = ttk.Frame(tframe)
        self.widgets["csl_editor_dropdown_frame"].grid(row=0, column=1, sticky=(W, E))
        self.widgets["csl_editor_dropdown_frame"].columnconfigure(0, weight=1)
        # self.widgets["csl_editor_dropdown"] = ttk.Combobox(tframe, values=["UNINITIALIZED"])
        # self.widgets["csl_editor_dropdown"].grid(
        #    row=0, column=1, sticky=(W, E), **TTKStyles.get_padding().right_entries
        # )
        ttk.Label(tframe, text="CSL file:").grid(
            row=1, column=0, sticky=W, **TTKStyles.get_padding().left_labels
        )
        self.widgets["csl_base_file_entry_value"] = StringVar(value="")
        self.widgets["csl_base_file_enty"] = ttk.Entry(tframe, textvariable=self.widgets["csl_base_file_entry_value"])
        self.widgets["csl_base_file_enty"].grid(
            row=1, column=1, sticky=(W, E), **TTKStyles.get_padding().right_entries
        )
        self.widgets["csl_select_button"] = ttk.Button(tframe, text="Select file")
        self.widgets["csl_select_button"].grid(
            row=2, columnspan=2, sticky=E, **TTKStyles.get_padding().right_entries
        )

    def _create_csl_buttons(self, tframe: ttk.Frame):
        buttonframe = ttk.Frame(tframe)
        buttonframe.grid(row=4, columnspan=2, sticky=E)
        self.widgets["csl_delete_button"] = ttk.Button(buttonframe, text="Delete entry")
        self.widgets["csl_delete_button"].grid(
            row=0, column=0, sticky=E, **TTKStyles.get_padding().inline_buttons
        )
        self.widgets["csl_delete_button"].state(["disabled"])
        self.widgets["csl_save_button"] = ttk.Button(buttonframe, text="Save entry")
        self.widgets["csl_save_button"].grid(
            row=0, column=1, sticky=E, **TTKStyles.get_padding().right_most_inline_button
        )

    def create(self, add_to: ttk.Frame) -> ttk.Frame:
        cframe = ttk.Frame(add_to)
        cframe.grid(column=0, row=0, sticky=(N, S, W, E))
        cframe.columnconfigure(0, weight=0)
        cframe.columnconfigure(1, weight=1)
        self._create_csl_file_selector(cframe)
        self._create_csl_buttons(cframe)
        return cframe



from pathlib import Path
from typing import Optional, Union
from abc import ABC, abstractmethod
from turandot import TurandotAssetException
from turandot.model import ConfigModel


class BaseAsset(ABC):
    """
    Base class for conversion assets
    Has a path, can read from it and can save exceptions if anything goes wrong
    """

    def __init__(self, path: Union[Path, str], expand: bool = False):
        self.path: Path = Path(path)
        self.directory: Optional[Path] = None
        self.content: str = ""
        self.exception: Optional[Exception] = None
        if expand:
            try:
                self.expand()
            except Exception as e:
                self.exception = e

    @abstractmethod
    def expand(self):
        """Create complete object with asset content"""
        pass

    def _read_path(self):
        """Fill path and directory attribute, throw exception if not found"""
        if str(self.path) == "":
            raise TurandotAssetException("Asset path must not be empty".format(self.path.name))
        if not self.path.is_file():
            raise TurandotAssetException("Not a file: {}".format(self.path.name))
        self.directory = self.path.parent

    def _read_content(self):
        """Read asset file content"""
        enc = ConfigModel().get_key(["general", "encoding"], default="utf8")
        with self.path.open('r', encoding=enc) as f:
            self.content = f.read()
